/*
 * @(#)EzLicenseInfo.h    1.00   02/21/02
 *
 * Copyright (c) 2002-Present Agilis Software LLC, Santa Clara, CA, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Agilis Software LLC ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only
 * in accordance with the terms of the license agreement you entered
 * into with Agilis Software LLC.
 */

/**
* The API manages the state of the information pertaining to 
* a license key and provides methods to check a license key.
*
* How to use for checking a license key in an ISV product: 
* 1. Allocate a license context:
*       void *licenseInfo = allocLicenseInfo();
* 2. Check the license key: 
*		int warningBitmap = checkLicenseKey(licenseInfo, strEncodedLicenseKey, etc.);  
*       The key state information is also set as a result, and may
*       be queried for specific key attributes.
* 3. If no exception (returned value > 0), the key is good, and the 
*    accessors for the key state can be invoked to obtain the decomposed
*    info such as quota limit, expiration date, concurrent-user
*	  limit, etc. 
* 4. If the returned warning bit map is non-zero, appropriate
*    warning messages / alerts may be issued by the ISV
*    application to the end user informing them of impending
*    expirations, quota exhaustion, etc.
* 5. Free the license context:
*    freeLicenseInfo(licenseInfo);
*
* Changes for 2.0:
* 1. If OS enforcement: user / host / net arg is optional
* 2. Windows: OS enforcement is based on case-insensitive
*    comparison.
* 3. Custom key handler is invoked after entire key is
*    assembled and irrespective of existence of custom
*    key value.
* 4. Allow for small (6-hr) timestamp discrepancies when
*    checking system clock.
* 5. System-clock-change-check: hard-enforce w/ file in
*    user home dir.
* 6. Expiration dates: base on absolute timestamps.
* 7. Context initialization for factory methods.
*
* New in 2.0:
* 1. Additional key content: key version, application password
* 2. Private-key-encrypted custom key, cookie and options if
*    application password specified.
* 3. Product name and application password public key argument 
*    for check API calls.
* 4. Visible additional key content: key version.
* 5. Automatic, secure application state management in key cookie 
*    using secure API extensions.  Quota consumption is built-in,
*    and in addition the application can store and update arbitrary
*    application-managed state in the key cookie.
* 6. Optional disabling of system clock checking for perpetual
*    keys (enabled by default).  If enabled, or if a key is time
*    limited, a cross-check is made against a hidden file and
*    (for the secure API's) the key cookie's timestamp.  This
*    applies to all flavors of license key checking API calls.
* 7. Optional hidden-file location specification.  By default,
*    the hidden file is located at the current directory (".").
*
* New in 2.5:
* 1. Library version and status message introspection functions
* 2. Changed semantics for disabling of system clock checking to
*    encompass time-limited licenses as well.
* 3. Added API call to set time tolerance.
*
*/

#ifndef __EZLICENSEINFO_H
#define  __EZLICENSEINFO_H

#include <IlsPlat.h>
#include <ezLicStatusCodes.h>
#include <ezLicLimits.h>
#include <ezLicCustomKeyHandler.h>

#ifdef __cplusplus
extern "C" {
#endif


/* CONSTANTS */


/**
* Runtime library normalized major version number
*/
#define EZLM_MAJOR_VERSION_NUMBER   25

/**
* Runtime library normalized minor version number
*/
#define EZLM_MINOR_VERSION_NUMBER   11


/**
* Previous license key version
*/
#define	EZLIC_KEY_VSN	20

/**
* Current new-feature license key version
*/
#define	EZLIC_KEY_VSN_CURR  25

/**
* Current new-feature secure license key version
*/
#define	EZLIC_KEY_VSN_CURR_S	26

/**
* Future license key version
*/
#define	EZLIC_KEY_VSN_NEXT	30

/**
* License models:
*/
/**
* 1. Single-user license model
*/
#define EZLIC_MODEL_USER 1
/**
* 2. Server license model
*/
#define EZLIC_MODEL_SERVER 2

/**
* Single-user license model: license types
*/
/**
* 1. User-locked
*/
#define EZLIC_TYPE_U_USER  1
/**
* 2. Node-locked
*/
#define EZLIC_TYPE_U_NODE  2
/**
* 3. Floating
*/
#define EZLIC_TYPE_U_FLOAT 3

/**
* Server license model : license types
*/
/**
* 1. Concurrent user
*/
#define EZLIC_TYPE_SVR_CONC  1
/**
* 2. CPU count
*/
#define EZLIC_TYPE_SVR_CPU  2
/**
* 3. CPU MHZ
*/
#define EZLIC_TYPE_SVR_MHZ 3
/**
* 4. Named-user
*/
#define EZLIC_TYPE_SVR_NMU 4

/**
* License modes - intended to be OR'ed into a bit map
*/
/**
* Time limited
*/
#define EZLIC_MODE_TIME 1
/**
* Metered
*/
#define EZLIC_MODE_METERED 2
/**
* Option-enabling
*/
#define EZLIC_MODE_OPTIONS 4
/**
* Custom key
*/
#define EZLIC_MODE_CUSTOM_KEY 8
/**
* Custom cookie
*/
#define EZLIC_MODE_CUSTOM_COOKIE 16


/**
* Default time tolerance in hours
*/
#define EZLIC_DEFAULT_TIME_TOLERANCE_HRS    6


/* VERSION ETC INFO */


/**
* Get version information string
* @param majorVersionBuffer buffer to receive major version number
* @param majorVersionBufferSize size of majorVersionBuffer
* @param minorVersionBuffer buffer to receive minor version number
* @param minorVersionBufferSize size of minorVersionBuffer
* @return: 0 if success, -1 if internal error or buffers too small
*/
DLLEXPORT int CALLING_CONVENTION getEzlmVersionInfoStr(
									char majorVersionBuffer[],
									size_t majorVersionBufferSize,
									char minorVersionBuffer[],
									size_t minorVersionBufferSize);

/**
* getStatusMsgStr:
* returns last status message from license info instance into supplied buffer
*	@param ezLicInfoInstance :	pointer to structure containing license info.
*   @param strBuffer: string buffer into which to return status message
*   @return: 0 if success, 1 if buffer overflow, -1 if internal error
*/
DLLEXPORT int CALLING_CONVENTION getStatusMsgStr(void *ezLicInfoInstance, 
										char strBuffer[],
										size_t bufferSize);

/**
* setTimeToleranceHrs:
* Set the time tolerance for system clock checking.
*   @param hrs: # of hrs of clock drift to tolerate.  -1 implies
*   system clock checking is nullified and any discrepance is
*   allowed.
*/
DLLEXPORT void CALLING_CONVENTION setTimeToleranceHrs(int tolerance);

/**
* getTimeToleranceHrs:
* Get the current setting for the time tolerance.
* @return current value of time tolerance.
*/
DLLEXPORT int CALLING_CONVENTION getTimeToleranceHrs();


/* LICENSE STATE MANAGEMENT */

/**
* Allocate license info
* @return pointer to allocated license info context
*/
DLLEXPORT void * CALLING_CONVENTION allocLicenseInfo();

/*
* Free license info
* @param licenseInfo: pointer to license info context to be freed
*/
DLLEXPORT void CALLING_CONVENTION freeLicenseInfo(void *licenseInfo);


/* LICENSE STATE ACCESSORS */


/**
* returns license key contained in license info context following
* a successful license key check.
* @param ezLicInfoInstance :	pointer to license info context
* @return: pointer to encoded license key string
*/
DLLEXPORT char * CALLING_CONVENTION getLicenseKey(void *ezLicInfoInstance);

/**
* returns license model code contained in license info context following
* a successful license key check.
* @param ezLicInfoInstance :	pointer to license info context
* @return license model code: one of EZLIC_MODEL_*
*/
DLLEXPORT int CALLING_CONVENTION getLicenseModelCode(void *ezLicInfoInstance);

/**
* returns license type code contained in license info context following
* a successful license key check.
* @param ezLicInfoInstance :	pointer to license info context
* @return license model code: one of EZLIC_TYPE_*
*/
DLLEXPORT int CALLING_CONVENTION getLicenseTypeCode(void *ezLicInfoInstance); 

/**
* returns license mode bitmap contained in license info context following
* a successful license key check.
* @param ezLicInfoInstance :	pointer to license info context
* @return license mode bitmap set per EZLIC_MODE_*.
*/
DLLEXPORT int CALLING_CONVENTION getLicenseModeBitmap(void *ezLicInfoInstance); 

/**
* returns custom key contained in the license info context following
* a successful license key check.
* @param ezLicInfoInstance :	pointer to license info context
* @return pointer to custom key string, which is an empty string if none.
*/
DLLEXPORT char * CALLING_CONVENTION getCustomKey(void *ezLicInfoInstance); 

/**
* returns custom cookie contained in the license info context following
* a successful license key check.
* @param ezLicInfoInstance :	pointer to license info context
* @return pointer to custom cookie string, which is an empty string if none.
*/
DLLEXPORT char * CALLING_CONVENTION getCustomCookie(void *ezLicInfoInstance); 

/**
* returns user / host name contained in the license info context following
* a successful license key check.
* @param ezLicInfoInstance :	pointer to license info context
* @return pointer to user / host name string, which is an empty string if none.
*/
DLLEXPORT char * CALLING_CONVENTION getUserHostNetName(void *ezLicInfoInstance); 

/**
* returns expiration date contained in the license info context following
* a successful license key check.
* @param ezLicInfoInstance :	pointer to license info context
* @return expiration date expressed in seconds since the Unix epoch, 0 if not time limited
*/
DLLEXPORT long CALLING_CONVENTION getExpirationTime(void *ezLicInfoInstance);

/**
* returns options contained in the license info context following
* a successful license key check.
* @param ezLicInfoInstance :	pointer to license info context
* @return pointer to options string, which is an empty string if none.
*/
DLLEXPORT char * CALLING_CONVENTION getOptions(void *ezLicInfoInstance); 

/**
* returns quota limit value contained in license info context following
* a successful license key check.
* @param ezLicInfoInstance :	pointer to license info context
* @return quota limit value, -1 if not a quota limited license.
*/
DLLEXPORT long CALLING_CONVENTION getQuotaValue(void *ezLicInfoInstance); 

/**
* returns usage limit value (cap on user limit / CPU limit etc.) 
* contained in license info context following a successful 
* license key check.
* @param ezLicInfoInstance :	pointer to license info context
* @return available limit value (cpu / CPU MHZ etc.) for
* the license key, -1 if no limit.
*/
DLLEXPORT long CALLING_CONVENTION getUsageValue(void *ezLicInfoInstance); 

/**
* returns warning bitmap contained in license info context following
* a successful license key check.
* @param ezLicInfoInstance :	pointer to license info context
* @return warning bitmap resulting from license key check, represented
* as a bitmap of the EZLIC_WARN_* bits defined in ezLicStatusCodes.h,
* 0 if no warnings.
*/
DLLEXPORT int CALLING_CONVENTION getWarningsBitmap(void *ezLicInfoInstance); 

/**
* Returns license key version contained in license info context following
* A value of -1 signifies a legacy value from keys generated w/o 
* version stamps.  The purpose of the key version is for managing
* compatibility moving forward beginning with EasyLicenser 2.0.
* A value == EZLIC_KEY_VSN signifies a current version.
* The run time library is guaranteed to work with key versions
* <= EZLIC_KEY_VSN and > EZLIC_KEY_VSN - 20.
* @return 2-digit key version number, first digit being major version.
* Example of return value: 20 for 2.0.
* a successful license key check.
* @param ezLicInfoInstance :	pointer to license info context
* @return key version represented as described above.
*/
DLLEXPORT int CALLING_CONVENTION getKeyVersion(void *ezLicInfoInstance); 

/**
* FOR INTERNAL USE ONLY
* Any other use can result in a core dump.
*/
DLLEXPORT char * CALLING_CONVENTION getAppPwd(void *ezLicInfoInstance);

/**
* RESERVED FOR FUTURE USE
* returns chained-key id contained in the license info context following
* a successful license key check.  Reserved for future use.
* @param ezLicInfoInstance :	pointer to license info context
* @return pointer to chained key id string, which is an empty string if none.
*/
DLLEXPORT char * CALLING_CONVENTION getChainedKeyId(void *ezLicInfoInstance);


/* 
 * ACCESSOR API CALLS FOR VISUAL BASIC APPLICATIONS: 
 * these take string buffer parameters instead of returning pointers. 
 */


/**
* getLicenseKeyStr:
* returns license key from license info instance into supplied buffer
*	@param ezLicInfoInstance :	pointer to structure containing license info.
*   @param strBuffer: string buffer into which to return license key
*   @return: 0 if success, 1 if buffer overflow, -1 if internal error
*/
DLLEXPORT int CALLING_CONVENTION getLicenseKeyStr(void *ezLicInfoInstance, 
										char strBuffer[],
										size_t bufferSize);



/**
* getCustomKeyStr:
* returns custom license key from license info instance into supplied buffer
*	@param ezLicInfoInstance :	pointer to structure containing license info.
*   @param strBuffer: string buffer into which to return custom key
*   @return: 0 if success, 1 if buffer overflow, -1 if internal error
*/
DLLEXPORT int CALLING_CONVENTION getCustomKeyStr(void *ezLicInfoInstance, 
										char strBuffer[],
										size_t bufferSize);



/**
* getCustomCookieStr:
* returns custom cookie from license info instance into supplied buffer
*	@param ezLicInfoInstance :	pointer to structure containing license info.
*   @param strBuffer: string buffer into which to return custom cookie
*   @return: 0 if success, 1 if buffer overflow, -1 if internal error
*/
DLLEXPORT int CALLING_CONVENTION getCustomCookieStr(void *ezLicInfoInstance, 
										char strBuffer[],
										size_t bufferSize);


/**
* getUserHostNetNameStr:
* returns user/host/network from license info instance into supplied buffer
*	@param ezLicInfoInstance :	pointer to structure containing license info.
*   @param strBuffer: string buffer into which to return user / host / network name
*   @return: 0 if success, 1 if buffer overflow, -1 if internal error
*/
DLLEXPORT int CALLING_CONVENTION getUserHostNetNameStr(void *ezLicInfoInstance, 
										char strBuffer[],
										size_t bufferSize);

/**
* getExpirationTimeStr:
* returns expiration date/time string from license info instance into supplied buffer.
* Expiration time is in GMT time zone.
*	@param ezLicInfoInstance :	pointer to structure containing license info.
*   @param strBuffer: string buffer into which to return expiration date string
*   @return: 0 if success, 1 if buffer overflow, -1 if internal error
*/
DLLEXPORT int CALLING_CONVENTION getExpirationTimeStr(void *ezLicInfoInstance, 
										char strBuffer[],
										size_t bufferSize);


/**
* getOptionsStr:
* returns options string from license info instance into supplied buffer
*	@param ezLicInfoInstance :	pointer to structure containing license info.
*   @param strBuffer: string buffer into which to return options string
*   @return: 0 if success, 1 if buffer overflow, -1 if internal error
*/
DLLEXPORT int CALLING_CONVENTION getOptionsStr(void *ezLicInfoInstance, 
										char strBuffer[],
										size_t bufferSize);


/**
* FOR INTERNAL USE ONLY
* Any other use can result in a core dump.
*/
DLLEXPORT int CALLING_CONVENTION getAppPwdStr(void *ezLicInfoInstance, 
										char strBuffer[],
										size_t bufferSize);


/**
* getChainedKeyIdStr:
* returns chained key id string from license info instance into supplied buffer
*	@param ezLicInfoInstance :	pointer to structure containing license info.
*   @param strBuffer: string buffer into which to return chained key id string
*   @return: 0 if success, 1 if buffer overflow, -1 if internal error
*/
DLLEXPORT int CALLING_CONVENTION getChainedKeyIdStr(void *ezLicInfoInstance, 
										char strBuffer[],
										size_t bufferSize);


/**
* getEnforce:
* returns indicator for whether user / host check is enforced.
*	@param ezLicInfoInstance :	pointer to structure containing license info.
*   @return: 0 if not enforced, 1 if enforced
*/
DLLEXPORT int CALLING_CONVENTION getEnforce(void *ezLicInfoInstance);



/* LICENSE STATE MUTATORS: FOR INTERNAL USE ONLY */


/* 
 * Note: using the mutators in your application will have no effect as
 * they are overridden internally as a result of the license key check
 * API call.  The immutable key completely determines the license terms
 * at run time.  The setters are presented here solely for internal
 * reasons.
 */

DLLEXPORT void CALLING_CONVENTION setLicenseKey(void *ezLicInfoInstance, const char* strLicensekey); 
DLLEXPORT void CALLING_CONVENTION setLicenseModelCode(void *ezLicInfoInstance, const int  iNewLicenseModelCode); 
DLLEXPORT void CALLING_CONVENTION setLicenseTypeCode(void *ezLicInfoInstance, const int  iNewLicenseTypeCode); 
DLLEXPORT void CALLING_CONVENTION setLicenseModeBitmap(void *ezLicInfoInstance, const int  iNewLicenseModeBitmap);
DLLEXPORT void CALLING_CONVENTION setCustomKey(void *ezLicInfoInstance, const char* strNewCustomKey); 
DLLEXPORT void CALLING_CONVENTION setCustomCookie(void *ezLicInfoInstance, const char* strNewCustomCookie); 
DLLEXPORT void CALLING_CONVENTION setUserHostNetName(void *ezLicInfoInstance, const char* strNewUserHostNetName); 
DLLEXPORT void CALLING_CONVENTION setExpirationTime(void *ezLicInfoInstance, const long lNewExpirationTime); 
DLLEXPORT void CALLING_CONVENTION setOptions(void *ezLicInfoInstance, const char* strNewOptions); 
DLLEXPORT void CALLING_CONVENTION setQuotaValue(void *ezLicInfoInstance, const long lNewQuotaValue); 
DLLEXPORT void CALLING_CONVENTION setUsageValue(void *ezLicInfoInstance, const long lNewUsageValue); 
DLLEXPORT void CALLING_CONVENTION setWarningsBitmap(void *ezLicInfoInstance, const int  iNewWarningsBitmap);
DLLEXPORT void CALLING_CONVENTION setChainedKeyId(void *ezLicInfoInstance, const char* strChainedKeyId); 
DLLEXPORT void CALLING_CONVENTION setKeyVersion(void *ezLicInfoInstance, const int  iKeyVersion);

/* APPLICATION STATE MANAGEMENT API's */

/**
* createKeyCookieSeed:
* Create a key cookie seed that is the first key cookie
* passed into the "checkLicenseKeySecure" API call
* by an application.  A system clock check is not performed,
* and no application state is managed in the encrypted cookie.
* An application invokes this
* method the first time it is executed after receiving
* a license key (typically during product installation),
* and saves the returned key cookie in
* persistent store.  Subsequently, the value in the persistent
* store is supplied to the "checkLicenseKeySecure" API call,
* which upon a successful license key check returns an updated
* key cookie that the application replaces the previous value
* with. The cookie mechanism automatically manages quota consumption 
* and optionally arbitrary application-managed state.
* The combination of the "createKeyCookieSeed" and
* "checkLicenseKeySecure" API's enables an application to be both
* spoof-proof and protect itself from hackers turning back
* the system clock to re-run quota- and time-limited copies of programs.
* IMPORTANT COMPATIBILITY NOTE: what used to be the "user host name"
* is now a "product name".  This is syntactictally compatible with previous 
* versions of EasyLicenser but does require that you provide the product
* name exactly as it was specified at the time of key generation.
* @param productName: the product name corresponding to the key
* @param strKeyCookieSeedBuf: string buffer to receive generated 
*                  key cookie string.
* @param keyCookieSeedBufsize: size of key cookie seed buffer.
* @return an EasyLicenser exception code: 0 if success,
* +ve value if warning, -ve EasyLicenser error code if error
*/
DLLEXPORT int CALLING_CONVENTION createKeyCookieSeed(
									const char *productName,
									char strKeyCookieSeedBuf[],
									size_t keyCookieSeedBufsize);

/**
* createKeyCookieSeed_c (complete version w/o hidden-file location control):
* Create a key cookie seed that is the first key cookie
* passed into the "checkLicenseKeySecure" API call
* by an application based on an application password, and
* specifying an application-managed state to be maintained
* securely in the key cookie.  An application invokes this
* method the first time it is executed after receiving
* a license key (typically during product installation),
* and saves the returned key cookie in
* persistent store.  Subsequently, the value in the persistent
* store is supplied to the "checkLicenseKeySecure" API call,
* which upon a successful license key check returns an updated
* key cookie that the application replaces the previous value
* with. The cookie mechanism automatically manages quota consumption 
* and optionally arbitrary application-managed state.
* The combination of the "createKeyCookieSeed" and
* "checkLicenseKeySecure" API's enables an application to be both
* spoof-proof and protect itself from hackers turning back
* the system clock to re-run quota- and time-limited copies of programs.
* IMPORTANT COMPATIBILITY NOTE: what used to be the "user host name"
* is now a "product name".  This is syntactictally compatible with previous 
* versions of EasyLicenser but does require that you provide the product
* name exactly as it was specified at the time of key generation.
* @param productName: the product name corresponding to the key
* @param appPassword: the public-key-encrypted form of the application
*   password that was defined at the time of key generation, and identical
*   to the application password parameter into the license key check API calls.
*   Specify NULL if the key is not protected with an application password.
* @param appState: pointer to a string representation of application state
*   that is to be encoded into the key cookie for secure storage.  Specify
*   NULL if no application state is to be saved.
* @param check: check whether the system clock or its checking mechanism has been tampered with
* @param strKeyCookieSeedBuf: string buffer to receive generated 
*   key cookie string.
* @param keyCookieSeedBufsize: size of key cookie seed buffer.
* @return an EasyLicenser exception code: 0 if success,
* +ve value if warning, -ve EasyLicenser error code if error
*/
DLLEXPORT int CALLING_CONVENTION createKeyCookieSeed_c(
									const char *productName,
									const char *appPassword,
									const char *appState,
									boolean check,
									char strKeyCookieSeedBuf[],
									size_t keyCookieSeedBufsize);

/**
* createKeyCookieSeed_ct (complete version with hidden-file location control):
* Create a key cookie seed that is the first key cookie
* passed into the "checkLicenseKeySecure" API call
* by an application based on an application password, and
* specifying an application-managed state to be maintained
* securely in the key cookie.  An application invokes this
* method the first time it is executed after receiving
* a license key (typically during product installation),
* and saves the returned key cookie in
* persistent store.  Subsequently, the value in the persistent
* store is supplied to the "checkLicenseKeySecure" API call,
* which upon a successful license key check returns an updated
* key cookie that the application replaces the previous value
* with. The cookie mechanism automatically manages quota consumption 
* and optionally arbitrary application-managed state.
* The combination of the "createKeyCookieSeed" and
* "checkLicenseKeySecure" API's enables an application to be both
* spoof-proof and protect itself from hackers turning back
* the system clock to re-run quota- and time-limited copies of programs.
* IMPORTANT COMPATIBILITY NOTE: what used to be the "user host name"
* is now a "product name".  This is syntactictally compatible with previous 
* versions of EasyLicenser but does require that you provide the product
* name exactly as it was specified at the time of key generation.
* @param productName: the product name corresponding to the key
* @param appPassword: the public-key-encrypted form of the application
*   password that was defined at the time of key generation, and identical
*   to the application password parameter into the license key check API calls.
*   Specify NULL if the key is not protected with an application password.
* @param appState: pointer to a string representation of application state
*   that is to be encoded into the key cookie for secure storage.  Specify
*   NULL if no application state is to be saved.
* @param check: check whether the system clock or its checking mechanism has been tampered with
* @param strKeyCookieSeedBuf: string buffer to receive generated 
*   key cookie string.
* @param keyCookieSeedBufsize: size of key cookie seed buffer.
* @param hfileDirectory: directory in which to place hidden file for timestamp checks.  
*   If null or empty string, it is the current directory.
* @return an EasyLicenser exception code: 0 if success,
* +ve value if warning, -ve EasyLicenser error code if error
*/
DLLEXPORT int CALLING_CONVENTION createKeyCookieSeed_ct(
									const char *productName,
									const char *appPassword,
									const char *appState,
									boolean check,
									char strKeyCookieSeedBuf[],
									size_t keyCookieSeedBufsize,
									const char *hfileDirectory);

/**
 * Clean up persistent structures associated with key cookie.
 * Typically invoked at uninstallation time.  Currently, just does
 * a validity check on the cookie without side effects.
 * @param keyCookie: last-valid key cookie
 * @param productName: product name
 * @param appPassword: application password, same as that specified
 *   to checkLicenseKeySecure() and createKeyCookieSeed().
 * @return an EasyLicenser exception code: 0 if success,
 * +ve value if warning, -ve EasyLicenser error code if error
 **/
DLLEXPORT int CALLING_CONVENTION cleanupKeyCookie(
									const char *keyCookie,
									const char *productName, 
									const char *appPassword);

/**
 * (Controlled hidden-file location and timestamp check version:)
 * Clean up persistent structures associated with key cookie.
 * Typically invoked at uninstallation time.  Currently, just does
 * a validity check on the cookie without side effects.
 * @param keyCookie: last-valid key cookie
 * @param productName: product name
 * @param appPassword: application password, same as that specified
 *   to checkLicenseKeySecure() and createKeyCookieSeed().
 * @param hfileDirectory: directory in which to expect hidden file.
 *   If NULL or an empty string, this is expected to be the current
 *   directory.
 * @check: whether to do a timestamp check against the system clock
 * @return an EasyLicenser exception code: 0 if success,
 * +ve value if warning, -ve EasyLicenser error code if error
 **/
DLLEXPORT int CALLING_CONVENTION cleanupKeyCookie_t(
									const char *keyCookie,
									const char *productName, 
									const char *appPassword,
									const char *hfileDirectory,
									boolean     check);

/**
 * Get application state stored in key cookie.
 * Invoked as needed during application execution, to
 * retrieve and process its extended licensing state.
 * @param keyCookie: last-valid key cookie
 * @param productName: product name
 * @param appPassword: application password, same as that specified
 * to checkLicenseKeySecure() and createKeyCookieSeed().
 * @param appStateBuffer: buffer to receive application state
 * @param appStateBufsize: size of application state buffer
 * @return an EasyLicenser exception code: 0 if success,
 * +ve value if warning, -ve EasyLicenser error code if error
 **/
DLLEXPORT int CALLING_CONVENTION getKeyCookieAppState(
									const char *keyCookie,
									const char *productName,
									const char *appPassword,
									char  appStateBuffer[],
									size_t appStateBufsize);

/**
 * Get quota-to-date stored in key cookie.
 * Invoked as needed during application execution, to
 * retrieve and process its metered quota consumption.
 * @param keyCookie: last-valid key cookie
 * @param productName: product name
 * @param appPassword: application password, same as that specified
 * to checkLicenseKeySecure() and createKeyCookieSeed().
 * @param pQuotaRet: pointer to long value to receive quota-to-date value.
 * 0 is deposited if not managed by key cookie or if not a metered license
 * @return an EasyLicenser exception code: 0 if success,
 * +ve value if warning, -ve EasyLicenser error code if error
 **/
DLLEXPORT int CALLING_CONVENTION getKeyCookieQuotaToDate(
									const char *keyCookie,
									const char *productName,
									const char *appPassword,
									long  *pQuotaRet);


/**
 * Get last-update timestamp stored in key cookie.
 * Invoked as needed during application execution, to
 * retrieve and process last-use timestamp.  Timestamp is
 * expressed in seconds since the epoch.
 * @param keyCookie: last-valid key cookie
 * @param productName: product name
 * @param appPassword: application password, same as that specified
 * to checkLicenseKeySecure() and createKeyCookieSeed().
 * @param pTimestampRet: pointer to time_t value to receive timestamp.
 * @return an EasyLicenser exception code: 0 if success,
 * +ve value if warning, -ve EasyLicenser error code if error
 **/
DLLEXPORT int CALLING_CONVENTION getKeyCookieTs(
									const char *keyCookie,
									const char *productName,
									const char *appPassword,
									time_t *pTimestampRet);

/* License Key Check API calls */

/**
* Check the specified single-user license key (legacy API call w/o access control).
*	@param ezLicInfoInstancep 		pointer to license info context.
*	@param strEncodedLicenseKey		The license key in encryptred form.
*	@param iDaysLeftThreshold :		threshold for # of days remaining in a time-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param lQuotaUsageToDate :		Total Number of license units used.
*	@param lQuotaThreshold :		Threshold for # of license units remaining in a usage-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param iDaysGracePeriod :		Number of days given as grace beyond the expiry date .
*	@param lQuotaGrace :			Number of license units given as grace.
*	@param strUserHostName			User or Host name.
*   @return an EasyLicenser exception code: 0 if success,
*   +ve value if warning, -ve EasyLicenser error code if error
*/
DLLEXPORT int CALLING_CONVENTION checkSingleUserLicenseKeyBasic (void* ezLicInfoInstancep, 
													    const char* 	strEncodedLicenseKey,
														const int		iDaysLeftThreshold,
														const long		lQuotaUsageToDate,
														const long		lQuotaThreshold,
														const int 		iDaysGracePeriod,
														const long		lQuotaGrace,
														const char*		strUserHostName);

/**
* Check the specified single-user license key (access-control version). This version of API
*   doesn't perform strict password check. If a key is generated with no password specification
*	in the product definition, the password-less key will pass the license check even if the
*   caller supplies an application password. For strict password check, use of the strict variant
*	of the function "checkSingleUserLicenseKeyBasic_px".
*	@param ezLicInfoInstancep 		pointer to license info context.
*	@param strEncodedLicenseKey		The license key in encryptred form.
*	@param iDaysLeftThreshold :		threshold for # of days remaining in a time-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param lQuotaUsageToDate :		Total Number of license units used.
*	@param lQuotaThreshold :		Threshold for # of license units remaining in a usage-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param iDaysGracePeriod :		Number of days given as grace beyond the expiry date .
*	@param lQuotaGrace :			Number of license units given as grace.
*	@param strUserHostName			User or Host name.
*   @param product:					product name.  Must match what's in the key if
*								    specified.  If null or an empty string, application
*									password can't be specified either.
*   @param appPassword:				application password's public key.
*                                   Must be specified if the key is access-controlled.
*   @return an EasyLicenser exception code: 0 if success,
*   +ve value if warning, -ve EasyLicenser error code if error
*/
DLLEXPORT int CALLING_CONVENTION checkSingleUserLicenseKeyBasic_p (void* ezLicInfoInstancep, 
													    const char* 	strEncodedLicenseKey,
														const int		iDaysLeftThreshold,
														const long		lQuotaUsageToDate,
														const long		lQuotaThreshold,
														const int 		iDaysGracePeriod,
														const long		lQuotaGrace,
														const char*		strUserHostName,
														const char*		product,
														const char*		appPassword);

/**
* Check the specified single-user license key (access-control version with strict password check).
*	@param ezLicInfoInstancep 		pointer to license info context.
*	@param strEncodedLicenseKey		The license key in encryptred form.
*	@param iDaysLeftThreshold :		threshold for # of days remaining in a time-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param lQuotaUsageToDate :		Total Number of license units used.
*	@param lQuotaThreshold :		Threshold for # of license units remaining in a usage-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param iDaysGracePeriod :		Number of days given as grace beyond the expiry date .
*	@param lQuotaGrace :			Number of license units given as grace.
*	@param strUserHostName			User or Host name.
*   @param product:					product name.  Must match what's in the key if
*								    specified.  If null or an empty string, application
*									password can't be specified either.
*   @param appPassword:				application password's public key.
*                                   Must be specified if the key is access-controlled.
*	@param strictPwdCheck			Whether the application password should be matched 
*									strictly against the application password used at the 
*									time of key generation.If product was not password 
*									protected, then application password can't be provided 
*									if strictPwdCheck is true.Else, it is not an error if an
*									application password is specified but the key was not
*									generated with an application password.
*   @return an EasyLicenser exception code: 0 if success,
*   +ve value if warning, -ve EasyLicenser error code if error
*/
DLLEXPORT int CALLING_CONVENTION checkSingleUserLicenseKeyBasic_px (void* ezLicInfoInstancep, 
													    const char* 	strEncodedLicenseKey,
														const int		iDaysLeftThreshold,
														const long		lQuotaUsageToDate,
														const long		lQuotaThreshold,
														const int 		iDaysGracePeriod,
														const long		lQuotaGrace,
														const char*		strUserHostName,
														const char*		product,
														const char*		appPassword,
														boolean			strictPwdCheck);

/**
* Check the server license key (legacy API call w/o access control).
*	@param ezLicInfoInstancep 		pointer to license info context.
*	@param strEncodedLicenseKey		The license key in encrypted form.
*	@param iDaysLeftThreshold :		threshold for # of days remaining in a time-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param lQuotaUsageToDate :		Total Number of license units used.
*	@param lQuotaThreshold :		Threshold for # of license units remaining in a usage-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param iDaysGracePeriod :		Number of days given as grace beyond the expiry date .
*	@param lQuotaGrace :			Number of license units given as grace.
*	@param strUserHostName			User or Host name.
*	@param lCurrUsageValue :		Total Number of users unsing the license.
*   @return an EasyLicenser exception code: 0 if success,
*   +ve value if warning, -ve EasyLicenser error code if error
*/
DLLEXPORT int CALLING_CONVENTION checkMultiUserLicenseKeyBasic (void*				ezLicInfoInstancep, 
				        							  const char* 		strEncodedLicenseKey,
													   const int 		iDaysLeftThreshold,
													   const long		lQuotaUsageToDate,
													   const long		lQuotaThreshold,
													   const int 		iDaysGracePeriod,
													   const long		lQuotaGrace, 
													   const char*		strUserHostName, 
													   const long		lCurrUsageValue);

/**
* Check the server license key (access control version). This version of API
*   doesn't perform strict password check. If a key is generated with no password specification
*	in the product definition, the password-less key will pass the license check even if the
*   caller supplies an application password. For strict password check, use of the strict variant
*	of the function "checkMultiUserLicenseKeyBasic_px".
*	@param ezLicInfoInstancep 		pointer to license info context.
*	@param strEncodedLicenseKey		The license key in encrypted form.
*	@param iDaysLeftThreshold :		threshold for # of days remaining in a time-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param lQuotaUsageToDate :		Total Number of license units used.
*	@param lQuotaThreshold :		Threshold for # of license units remaining in a usage-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param iDaysGracePeriod :		Number of days given as grace beyond the expiry date .
*	@param lQuotaGrace :			Number of license units given as grace.
*	@param strUserHostName			User or Host name.
*	@param lCurrUsageValue :		Total Number of users unsing the license.
*   @param product:					product name.  Must match what's in the key if
*								    specified.  If null or an empty string, application
*									password can't be specified either.
*   @param appPassword:				application password's public key.
*                                   Must be specified if the key is access-controlled.
*   @return an EasyLicenser exception code: 0 if success,
*   +ve value if warning, -ve EasyLicenser error code if error
*/
DLLEXPORT int CALLING_CONVENTION checkMultiUserLicenseKeyBasic_p(
													   void*			ezLicInfoInstancep, 
				        							   const char* 		strEncodedLicenseKey,
													   const int 		iDaysLeftThreshold,
													   const long		lQuotaUsageToDate,
													   const long		lQuotaThreshold,
													   const int 		iDaysGracePeriod,
													   const long		lQuotaGrace, 
													   const char*		strUserHostName, 
													   const long		lCurrUsageValue,
													   const char*		product,
													   const char*		appPassword);


/**
* Check the server license key (access control version with strict password check).
*	@param ezLicInfoInstancep 		pointer to license info context.
*	@param strEncodedLicenseKey		The license key in encrypted form.
*	@param iDaysLeftThreshold :		threshold for # of days remaining in a time-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param lQuotaUsageToDate :		Total Number of license units used.
*	@param lQuotaThreshold :		Threshold for # of license units remaining in a usage-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param iDaysGracePeriod :		Number of days given as grace beyond the expiry date .
*	@param lQuotaGrace :			Number of license units given as grace.
*	@param strUserHostName			User or Host name.
*	@param lCurrUsageValue :		Total Number of users unsing the license.
*   @param product:					product name.  Must match what's in the key if
*								    specified.  If null or an empty string, application
*									password can't be specified either.
*   @param appPassword:				application password's public key.
*                                   Must be specified if the key is access-controlled.
*	@param strictPwdCheck			Whether the application password should be matched 
*									strictly against the application password used at the 
*									time of key generation.If product was not password 
*									protected, then application password can't be provided 
*									if strictPwdCheck is true.Else,it is not an error if an
*									application password is specified but the key was not
*									generated with an application password.
*   @return an EasyLicenser exception code: 0 if success,
*   +ve value if warning, -ve EasyLicenser error code if error
*/
DLLEXPORT int CALLING_CONVENTION checkMultiUserLicenseKeyBasic_px(
													   void*			ezLicInfoInstancep, 
				        							   const char* 		strEncodedLicenseKey,
													   const int 		iDaysLeftThreshold,
													   const long		lQuotaUsageToDate,
													   const long		lQuotaThreshold,
													   const int 		iDaysGracePeriod,
													   const long		lQuotaGrace, 
													   const char*		strUserHostName, 
													   const long		lCurrUsageValue,
													   const char*		product,
													   const char*		appPassword,
													   boolean			strictPwdCheck);
/**
* (Full legacy version, no application password / access control / 
* hidden-file / timestamp check control:) 
* Check the specified license key against the provided info for key violations,
* and set the license context based on the key.  It is an error if the 
* application password doesn't match what's in the key.
* There is an application password mismatch if the key
* contains an application password and either no password
* is passed as a parameter or the corresponding public
* key doesn't match the supplied application password public key.
* It is not an error if an application password is specified but the
* key was not generated with an application password.For strict password check, 
*   use of the strict variant of the function "checkLicenseKey_ptx".
*	@param ezLicInfoInstancep 		pointer to license info context.
*	@param strEncodedLicenseKey		The license key in encryptred form.
*	@param customKeyHandler			Pointer to Custom key handler function.
*	@param customKeyContext			Pointer to Custom key context.
*	@param iDaysLeftThreshold :		threshold for # of days remaining in a time-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param lQuotaUsageToDate :		Total Number of license units used.
*	@param lQuotaThreshold :		Threshold for # of license units remaining in a usage-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param iDaysGracePeriod :		Number of days given as grace beyond the expiry date .
*	@param lQuotaGrace :			Number of license units given as grace.
*	@param strUserHostName			User or Host name.
*	@param lCurrUsageValue :		Total Number of users unsing the license.
*   @return an EasyLicenser exception code: 0 if success,
*   +ve value if warning, -ve EasyLicenser error code if error
*/
DLLEXPORT int CALLING_CONVENTION checkLicenseKey(void* 			ezLicInfoInstancep, 
									   const char*		strEncodedLicenseKey,	
									   ezLicCustomKeyHandler_t customKeyHandler,
									   void*			customKeyContext,
									   const int 		iDaysLeftThreshold, 
									   const long		lQuotaUsageToDate,     
									   const long		lQuotaThreshold, 
									   const int 		iDaysGracePeriod, 
									   const long		lQuotaGrace, 
									   const char*		strUserHostName, 
									   const long		lCurrUsageValue);


/**
* (Full version, with access control but no hidden file / timestamp check control:) 
* Check the specified license key against the provided info for key violations,
* and set the license context based on the key. This version of API
*   doesn't perform strict password check. If a key is generated with no password
*	specification in the product definition, the password-less key will pass the license 
*	check even if the caller supplies an application password. For strict password check,
*	use of the strict variant of the function "checkLicenseKey_px".
*	@param ezLicInfoInstancep 		pointer to license info context.
*	@param strEncodedLicenseKey		The license key in encryptred form.
*	@param customKeyHandler			Pointer to Custom key handler function.
*	@param customKeyContext			Pointer to Custom key context.
*	@param iDaysLeftThreshold :		threshold for # of days remaining in a time-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param lQuotaUsageToDate :		Total Number of license units used.
*	@param lQuotaThreshold :		Threshold for # of license units remaining in a usage-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param iDaysGracePeriod :		Number of days given as grace beyond the expiry date .
*	@param lQuotaGrace :			Number of license units given as grace.
*	@param strUserHostName			User or Host name.
*	@param lCurrUsageValue :		Total Number of users unsing the license.
*   @param product: 				product name.  Must match what's in the key if
*		    						specified.  If not specified, app password can't be
*          							specified either.
*   @param appPassword: 			application password's public key.  
*          							One must be supplied if the key contains a password.
*   @return an EasyLicenser exception code: 0 if success,
*   +ve value if warning, -ve EasyLicenser error code if error
*/
DLLEXPORT int CALLING_CONVENTION checkLicenseKey_p(
									   void* 			ezLicInfoInstancep, 
									   const char*		strEncodedLicenseKey,	
									   ezLicCustomKeyHandler_t customKeyHandler,
									   void*			customKeyContext,
									   const int 		iDaysLeftThreshold, 
									   const long		lQuotaUsageToDate,     
									   const long		lQuotaThreshold, 
									   const int 		iDaysGracePeriod, 
									   const long		lQuotaGrace, 
									   const char*		strUserHostName, 
									   const long		lCurrUsageValue,
									   const char*		product,
									   const char*		appPassword);


/**
* (Full version, with access control but no hidden file / timestamp check control with
*	strict password check:) 
* Check the specified license key against the provided info for key violations,
* and set the license context based on the key. 
*	@param ezLicInfoInstancep 		pointer to license info context.
*	@param strEncodedLicenseKey		The license key in encryptred form.
*	@param customKeyHandler			Pointer to Custom key handler function.
*	@param customKeyContext			Pointer to Custom key context.
*	@param iDaysLeftThreshold :		threshold for # of days remaining in a time-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param lQuotaUsageToDate :		Total Number of license units used.
*	@param lQuotaThreshold :		Threshold for # of license units remaining in a usage-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param iDaysGracePeriod :		Number of days given as grace beyond the expiry date .
*	@param lQuotaGrace :			Number of license units given as grace.
*	@param strUserHostName			User or Host name.
*	@param lCurrUsageValue :		Total Number of users unsing the license.
*   @param product: 				product name.  Must match what's in the key if
*		    						specified.  If not specified, app password can't be
*          							specified either.
*   @param appPassword: 			application password's public key.  
*          							One must be supplied if the key contains a password.
*	@param strictPwdCheck			Whether the application password should be matched 
*									strictly against the application password used at the 
*									time of key generation.If product was not password 
*									protected, then application password can't be provided 
*									if strictPwdCheck is true.Else, it is not an error if an
*									application password is specified but the key was not
*									generated with an application password.
*   @return an EasyLicenser exception code: 0 if success,
*   +ve value if warning, -ve EasyLicenser error code if error
*/
DLLEXPORT int CALLING_CONVENTION checkLicenseKey_px(
									   void* 			ezLicInfoInstancep, 
									   const char*		strEncodedLicenseKey,	
									   ezLicCustomKeyHandler_t customKeyHandler,
									   void*			customKeyContext,
									   const int 		iDaysLeftThreshold, 
									   const long		lQuotaUsageToDate,     
									   const long		lQuotaThreshold, 
									   const int 		iDaysGracePeriod, 
									   const long		lQuotaGrace, 
									   const char*		strUserHostName, 
									   const long		lCurrUsageValue,
									   const char*		product,
									   const char*		appPassword,
									   boolean			strictPwdCheck);


/**
* (Full version, with access control and hidden file / timestamp check control:) 
* Check the specified license key against the provided info for key violations,
* and set the license context based on the key. This version of API
*   doesn't perform strict password check. If a key is generated with no password
*	specification in the product definition, the password-less key will pass the license 
*	check even if the caller supplies an application password. For strict password check,
*	use of the strict variant of the function "checkLicenseKey_ptx".
*	@param ezLicInfoInstancep 		pointer to license info context.
*	@param strEncodedLicenseKey		The license key in encryptred form.
*	@param customKeyHandler			Pointer to Custom key handler function.
*	@param customKeyContext			Pointer to Custom key context.
*	@param iDaysLeftThreshold :		threshold for # of days remaining in a time-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param lQuotaUsageToDate :		Total Number of license units used.
*	@param lQuotaThreshold :		Threshold for # of license units remaining in a usage-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param iDaysGracePeriod :		Number of days given as grace beyond the expiry date .
*	@param lQuotaGrace :			Number of license units given as grace.
*	@param strUserHostName			User or Host name.
*	@param lCurrUsageValue :		Total Number of users unsing the license.
*   @param product: 				product name.  Must match what's in the key if
*		    						specified.  If not specified, app password can't be
*          							specified either.
*   @param appPassword: 			application password's public key.  
*          							One must be supplied if the key contains a password.
*   @param hfileDirectory:			directory in which to expect hidden file.
*									If NULL or an empty string, this is expected to be the
*									current directory.
*   @param check:					whether to unconditionally do a timestamp check against 
*									the system clock.If false, the timestamp check is done 
*									only if a time limited key is detected.
*   @return an EasyLicenser exception code: 0 if success,
*   +ve value if warning, -ve EasyLicenser error code if error
*/
DLLEXPORT int CALLING_CONVENTION checkLicenseKey_pt(
									   void* 			ezLicInfoInstancep, 
									   const char*		strEncodedLicenseKey,	
									   ezLicCustomKeyHandler_t customKeyHandler,
									   void*			customKeyContext,
									   const int 		iDaysLeftThreshold, 
									   const long		lQuotaUsageToDate,     
									   const long		lQuotaThreshold, 
									   const int 		iDaysGracePeriod, 
									   const long		lQuotaGrace, 
									   const char*		strUserHostName, 
									   const long		lCurrUsageValue,
									   const char*		product,
									   const char*		appPassword,
									   const char*		hfileDirectory,
									   boolean			check);


/**
* (Full version, with access control and hidden file / timestamp check control and strict
*	password check:) 
* Check the specified license key against the provided info for key violations,
* and set the license context based on the key.
*	@param ezLicInfoInstancep 		pointer to license info context.
*	@param strEncodedLicenseKey		The license key in encryptred form.
*	@param customKeyHandler			Pointer to Custom key handler function.
*	@param customKeyContext			Pointer to Custom key context.
*	@param iDaysLeftThreshold :		threshold for # of days remaining in a time-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param lQuotaUsageToDate :		Total Number of license units used.
*	@param lQuotaThreshold :		Threshold for # of license units remaining in a usage-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param iDaysGracePeriod :		Number of days given as grace beyond the expiry date .
*	@param lQuotaGrace :			Number of license units given as grace.
*	@param strUserHostName			User or Host name.
*	@param lCurrUsageValue :		Total Number of users unsing the license.
*   @param product: 				product name.  Must match what's in the key if
*		    						specified.  If not specified, app password can't be
*          							specified either.
*   @param appPassword: 			application password's public key.  
*          							One must be supplied if the key contains a password.
*   @param hfileDirectory:			directory in which to expect hidden file.
*								    If NULL or an empty string, this is expected to be the 
*									current directory.
*   @parm check:					whether to unconditionally do a timestamp check against 
*									the system clock.If false, the timestamp check is done 
*									only if a time limited key is detected.
*	@param strictPwdCheck			Whether the application password should be matched 
*									strictly against the application password used at the 
*									time of key generation.If product was not password 
*									protected, then application password can't be provided 
*									if strictPwdCheck is true.Else, it is not an error if an
*									application password is specified but the key was not
*									generated with an application password.
*   @return an EasyLicenser exception code: 0 if success,
*   +ve value if warning, -ve EasyLicenser error code if error
*/
DLLEXPORT int CALLING_CONVENTION checkLicenseKey_ptx(
									   void* 			ezLicInfoInstancep, 
									   const char*		strEncodedLicenseKey,	
									   ezLicCustomKeyHandler_t customKeyHandler,
									   void*			customKeyContext,
									   const int 		iDaysLeftThreshold, 
									   const long		lQuotaUsageToDate,     
									   const long		lQuotaThreshold, 
									   const int 		iDaysGracePeriod, 
									   const long		lQuotaGrace, 
									   const char*		strUserHostName, 
									   const long		lCurrUsageValue,
									   const char*		product,
									   const char*		appPassword,
									   const char*		hfileDirectory,
									   boolean			check,
									   boolean			strictPwdCheck);

/**
* (Basic secure version (legacy API): no application passwords / application state management /
* hidden-file / timestamp-check control) 
* Check the specified license key against the provided info for key violations,
* and set the context based on the key.
*	@param ezLicInfoInstancep 		pointer to license info context.
*	@param strEncodedLicenseKey		The license key in encryptred form.
*   @param strKeyCookie:			the key cookie that was returned
*					          		from the previous call to this method for 
*          							this key, or from the createKeyCookieSeed() 
*          							method if none.
*   @param strNewKeyCookieBuf:		Pointer to buffer to receive new key cookie value.
*   @param newKeyCookieBufsize:     Size of new key cookie buffer
*	@param customKeyHandler			Pointer to Custom key handler function.
*	@param customKeyContext			Pointer to Custom key context.
*	@param iDaysLeftThreshold :		threshold for # of days remaining in a time-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param lQuotaUsageToDate :		Total Number of license units used.
*	@param lQuotaThreshold :		Threshold for # of license units remaining in a usage-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param iDaysGracePeriod :		Number of days given as grace beyond the expiry date .
*	@param lQuotaGrace :			Number of license units given as grace.
*	@param strUserHostName			User or Host name.
*	@param lCurrUsageValue :		Total Number of users unsing the license.
*   @return an EasyLicenser exception code: 0 if success,
*   +ve value if warning, -ve EasyLicenser error code if error
*/
DLLEXPORT int CALLING_CONVENTION checkLicenseKeySecure(
									   void* 			ezLicInfoInstancep, 
									   const char*		strEncodedLicenseKey,	
									   const char*		strKeyCookie,
									   char				strNewKeyCookieBuf[],
									   size_t			strNewKeyCookieBufsize,
									   ezLicCustomKeyHandler_t customKeyHandler,
									   void*			customKeyContext,
									   const int 		iDaysLeftThreshold, 
									   const long		lQuotaUsageToDate,     
									   const long		lQuotaThreshold, 
									   const int 		iDaysGracePeriod, 
									   const long		lQuotaGrace, 
									   const char*		strUserHostName, 
									   const long		lCurrUsageValue);


/**
* (Full secure version: application passwords and application state management
* but no hidden file / timestamp check control) 
* Check the specified license key against the provided info for key violations,
* and set the context based on the key.
* This version matches product name and application password, and also accepts 
* an application context for embedding into the key cookie.
* This version of API doesn't perform strict password check. If a key is generated with no
* password specification in the product definition, the password-less key will pass the license 
*	check even if the caller supplies an application password. For strict password check,
*	use of the strict variant of the function "checkLicenseKeySecure_sx".
*	@param ezLicInfoInstancep 		pointer to license info context.
*	@param strEncodedLicenseKey		The license key in encryptred form.
*   @param strKeyCookie:			the key cookie that was returned
*					          		from the previous call to this method for 
*          							this key, or from the createKeyCookieSeed() 
*          							method if none.
*   @param strNewKeyCookieBuf:		Pointer to buffer to receive new key cookie value.
*   @param newKeyCookieBufsize:     Size of new key cookie buffer
*	@param customKeyHandler			Pointer to Custom key handler function.
*	@param customKeyContext			Pointer to Custom key context.
*	@param iDaysLeftThreshold :		threshold for # of days remaining in a time-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param lQuotaUsageToDate :		Total Number of license units used.
*	@param lQuotaThreshold :		Threshold for # of license units remaining in a usage-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param iDaysGracePeriod :		Number of days given as grace beyond the expiry date .
*	@param lQuotaGrace :			Number of license units given as grace.
*	@param strUserHostName			User or Host name.
*	@param lCurrUsageValue :		Total Number of users unsing the license.
*   @param product: 				product name.  Must match what's in the key if
*		    						specified.  If not specified, app password can't be
*          							specified either.
*   @param appPassword: 			application password's public key.  
*          							One must be supplied if the key contains a password.
*   @param appState:				pointer to string representation of application state,
*								    NULL if application state is not to be updated.
*   @return an EasyLicenser exception code: 0 if success,
*   +ve value if warning, -ve EasyLicenser error code if error
*/
DLLEXPORT int CALLING_CONVENTION checkLicenseKeySecure_s(
									   void* 			ezLicInfoInstancep, 
									   const char*		strEncodedLicenseKey,	
									   const char*		strKeyCookie,
									   char				strNewKeyCookieBuf[],
									   size_t			strNewKeyCookieBufsize,
									   ezLicCustomKeyHandler_t customKeyHandler,
									   void*			customKeyContext,
									   const int 		iDaysLeftThreshold, 
									   const long		lQuotaUsageToDate,     
									   const long		lQuotaThreshold, 
									   const int 		iDaysGracePeriod, 
									   const long		lQuotaGrace, 
									   const char*		strUserHostName, 
									   const long		lCurrUsageValue,
									   const char*		product,
									   const char*		appPassword,
									   const char*		appState);

/**
* (Full secure version: application passwords and application state management
* but no hidden file / timestamp check control and strict password check) 
* Check the specified license key against the provided info for key violations,
* and set the context based on the key.
* This version matches product name and application password, and also accepts 
* an application context for embedding into the key cookie.
*	@param ezLicInfoInstancep 		pointer to license info context.
*	@param strEncodedLicenseKey		The license key in encryptred form.
*   @param strKeyCookie:			the key cookie that was returned
*					          		from the previous call to this method for 
*          							this key, or from the createKeyCookieSeed() 
*          							method if none.
*   @param strNewKeyCookieBuf:		Pointer to buffer to receive new key cookie value.
*   @param newKeyCookieBufsize:     Size of new key cookie buffer
*	@param customKeyHandler			Pointer to Custom key handler function.
*	@param customKeyContext			Pointer to Custom key context.
*	@param iDaysLeftThreshold :		threshold for # of days remaining in a time-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param lQuotaUsageToDate :		Total Number of license units used.
*	@param lQuotaThreshold :		Threshold for # of license units remaining in a usage-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param iDaysGracePeriod :		Number of days given as grace beyond the expiry date .
*	@param lQuotaGrace :			Number of license units given as grace.
*	@param strUserHostName			User or Host name.
*	@param lCurrUsageValue :		Total Number of users unsing the license.
*   @param product: 				product name.  Must match what's in the key if
*		    						specified.  If not specified, app password can't be
*          							specified either.
*   @param appPassword: 			application password's public key.  
*          							One must be supplied if the key contains a password.
*   @param appState:				pointer to string representation of application state,
*								    NULL if application state is not to be updated.
*	@param strictPwdCheck			Whether the application password should be matched 
*									strictly against the application password used at the 
*									time of key generation.If product was not password 
*									protected, then application password can't be provided 
*									if strictPwdCheck is true.Else, it is not an error if an
*									application password is specified but the key was not
*									generated with an application password.
*   @return an EasyLicenser exception code: 0 if success,
*   +ve value if warning, -ve EasyLicenser error code if error
*/
DLLEXPORT int CALLING_CONVENTION checkLicenseKeySecure_sx(
									   void* 			ezLicInfoInstancep, 
									   const char*		strEncodedLicenseKey,	
									   const char*		strKeyCookie,
									   char				strNewKeyCookieBuf[],
									   size_t			strNewKeyCookieBufsize,
									   ezLicCustomKeyHandler_t customKeyHandler,
									   void*			customKeyContext,
									   const int 		iDaysLeftThreshold, 
									   const long		lQuotaUsageToDate,     
									   const long		lQuotaThreshold, 
									   const int 		iDaysGracePeriod, 
									   const long		lQuotaGrace, 
									   const char*		strUserHostName, 
									   const long		lCurrUsageValue,
									   const char*		product,
									   const char*		appPassword,
									   const char*		appState,
									   boolean			strictPwdCheck);


/**
* (Full secure version: application passwords and application state management
*  with hidden file / timestamp check control) 
* Check the specified license key against the provided info for key violations,
* and set the context based on the key.
* This version matches product name and application password, and also accepts 
* an application context for embedding into the key cookie.
* This version of API doesn't perform strict password check. If a key is generated with
* no password specification in the product definition, the password-less key will pass the 
* license check even if the caller supplies an application password. For strict password
* check,use of the strict variant of the function "checkLicenseKeySecure_stx".
*	@param ezLicInfoInstancep 		pointer to license info context.
*	@param strEncodedLicenseKey		The license key in encryptred form.
*   @param strKeyCookie:			the key cookie that was returned
*          							from the previous call to this method for 
*          							this key, or from the createKeyCookieSeed() 
*          							method if none.
*   @param strNewKeyCookieBuf:		Pointer to buffer to receive new key cookie value.
*   @param newKeyCookieBufsize:     Size of new key cookie buffer
*	@param customKeyHandler			Pointer to Custom key handler function.
*	@param customKeyContext			Pointer to Custom key context.
*	@param iDaysLeftThreshold :		threshold for # of days remaining in a time-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param lQuotaUsageToDate :		Total Number of license units used.
*	@param lQuotaThreshold :		Threshold for # of license units remaining in a usage-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param iDaysGracePeriod :		Number of days given as grace beyond the expiry date .
*	@param lQuotaGrace :			Number of license units given as grace.
*	@param strUserHostName			User or Host name.
*	@param lCurrUsageValue :		Total Number of users unsing the license.
*   @param product: 				product name.  Must match what's in the key if
*		    						specified.  If not specified, app password can't be
*          							specified either.
*   @param appPassword: 			application password's public key.  
*          							One must be supplied if the key contains a password.
*   @param appState:				pointer to string representation of application state,
*								    NULL if application state is not to be updated.
*   @param hfileDirectory:			directory in which to manage hidden timestamp file,
*									defaults to current directory if NULL or empty string.
*   @param check:					whether to unconditionally check for tampering of
*									system clock.  If false, the check is made only if
*									the key is detected to be time limited.
*   @return an EasyLicenser exception code: 0 if success,
*   +ve value if warning, -ve EasyLicenser error code if error
*/
DLLEXPORT int CALLING_CONVENTION checkLicenseKeySecure_st(
									   void* 			ezLicInfoInstancep, 
									   const char*		strEncodedLicenseKey,	
									   const char*		strKeyCookie,
									   char				strNewKeyCookieBuf[],
									   size_t			strNewKeyCookieBufsize,
									   ezLicCustomKeyHandler_t customKeyHandler,
									   void*			customKeyContext,
									   const int 		iDaysLeftThreshold, 
									   const long		lQuotaUsageToDate,     
									   const long		lQuotaThreshold, 
									   const int 		iDaysGracePeriod, 
									   const long		lQuotaGrace, 
									   const char*		strUserHostName, 
									   const long		lCurrUsageValue,
									   const char*		product,
									   const char*		appPassword,
									   const char*		appState,
									   const char*		hfileDirectory,
									   boolean			check);

/**
* (Full secure version: application passwords and application state management
*  with hidden file / timestamp check control) 
* Check the specified license key against the provided info for key violations,
* and set the context based on the key.
* This version matches product name and application password, and also accepts 
* an application context for embedding into the key cookie.
*	@param ezLicInfoInstancep 		pointer to license info context.
*	@param strEncodedLicenseKey		The license key in encryptred form.
*   @param strKeyCookie:			the key cookie that was returned
*          							from the previous call to this method for 
*          							this key, or from the createKeyCookieSeed() 
*          							method if none.
*   @param strNewKeyCookieBuf:		Pointer to buffer to receive new key cookie value.
*   @param newKeyCookieBufsize:     Size of new key cookie buffer
*	@param customKeyHandler			Pointer to Custom key handler function.
*	@param customKeyContext			Pointer to Custom key context.
*	@param iDaysLeftThreshold :		threshold for # of days remaining in a time-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param lQuotaUsageToDate :		Total Number of license units used.
*	@param lQuotaThreshold :		Threshold for # of license units remaining in a usage-limited license in order
*										to trigger an "about-to-expire" warning.
*	@param iDaysGracePeriod :		Number of days given as grace beyond the expiry date .
*	@param lQuotaGrace :			Number of license units given as grace.
*	@param strUserHostName			User or Host name.
*	@param lCurrUsageValue :		Total Number of users unsing the license.
*   @param product: 				product name.  Must match what's in the key if
*		    						specified.  If not specified, app password can't be
*          							specified either.
*   @param appPassword: 			application password's public key.  
*          							One must be supplied if the key contains a password.
*   @param appState:				pointer to string representation of application state,
*								    NULL if application state is not to be updated.
*   @param hfileDirectory:			directory in which to manage hidden timestamp file,
*									defaults to current directory if NULL or empty string.
*   @param check:					whether to unconditionally check for tampering of
*									system clock.  If false, the check is made only if
*									the key is detected to be time limited.
*	@param strictPwdCheck			Whether the application password should be matched 
*									strictly against the application password used at the 
*									time of key generation.If product was not password 
*									protected, then application password can't be provided 
*									if strictPwdCheck is true.Else, it is not an error if an
*									application password is specified but the key was not
*									generated with an application password.
*   @return an EasyLicenser exception code: 0 if success,
*   +ve value if warning, -ve EasyLicenser error code if error
*/
DLLEXPORT int CALLING_CONVENTION checkLicenseKeySecure_stx(
									   void* 			ezLicInfoInstancep, 
									   const char*		strEncodedLicenseKey,	
									   const char*		strKeyCookie,
									   char				strNewKeyCookieBuf[],
									   size_t			strNewKeyCookieBufsize,
									   ezLicCustomKeyHandler_t customKeyHandler,
									   void*			customKeyContext,
									   const int 		iDaysLeftThreshold, 
									   const long		lQuotaUsageToDate,     
									   const long		lQuotaThreshold, 
									   const int 		iDaysGracePeriod, 
									   const long		lQuotaGrace, 
									   const char*		strUserHostName, 
									   const long		lCurrUsageValue,
									   const char*		product,
									   const char*		appPassword,
									   const char*		appState,
									   const char*		hfileDirectory,
									   boolean			check,
									   boolean			strictPwdCheck);


/**
 * Drop hidden file located at specified directory.
 */
boolean CALLING_CONVENTION dropHfile(const char *hfileDirectory);


/**
 * Make an encrypted library digest (signature) for the
 * EasyLicenser runtime library file in the runtime environment.
 * @param password: application password encrypted with the
 *    product's public key.
 * @param strSignatureBuf: buffer to receive signature
 * @param signatureBufsize: size of signature buffer
 * @param strEncodedSigBuf: buffer to receive public-key-encrypted signature
 * @param encodedSigBufsize: size of public-key-encrypted signature buffer
 * @return status: 0 if success, -1 otherwise.
 **/
DLLEXPORT int CALLING_CONVENTION makeEzlmLibDigest(
									 const char *password,
									 char strSignatureBuf[],
									 size_t signatureBufsize,
									 char strEncodedSigBuf[],
									 size_t encodedSigBufsize);


#ifdef __cplusplus
}
#endif

#endif

	
